@@ -23,24 +23,24 @@
 # *
 # **************************************************************************
 import logging
+logger = logging.getLogger(__name__)
 import csv
 
 from emtable import Table
-from pwem import ALIGN_NONE, ALIGN_2D, ALIGN_PROJ
+from pwem import ALIGN_NONE
 from pwem.convert.headers import fixVolume
 from pwem.objects import Transform
 from pyworkflow.object import Integer, Float
 from pyworkflow.utils import getParentFolder, yellowStr, createLink, makePath
 from relion.convert import OpticsGroups
 from reliontomo.constants import *
-import pwem.convert.transformations as tfs
 import numpy as np
 from os.path import join, basename
 from reliontomo.convert.convertBase import (checkSubtomogramFormat,
                                             getTransformInfoFromCoordOrSubtomo,
                                             WriterTomo, ReaderTomo, getTransformMatrixFromRow)
 from reliontomo.objects import RelionPSubtomogram
-from tomo.constants import BOTTOM_LEFT_CORNER, TR_SCIPION, TR_RELION, SCIPION
+from tomo.constants import BOTTOM_LEFT_CORNER, TR_RELION, SCIPION
 from tomo.objects import Coordinate3D, SubTomogram, TomoAcquisition
 
 logger = logging.getLogger(__name__)
@@ -80,15 +80,20 @@ class Writer(WriterTomo):
         # Write the STAR file
         tsTable.write(outStarFileName)
 
-    def coordinates2Star(self, coordSet, subtomosStar, sRate=1, coordsScale=1):
+    def coordinates2Star(self, coordSet, subtomosStar, whitelist, sRate=1, coordsScale=1):
         """Input coordsScale is used to scale the coordinates so they are expressed in bin 1, as expected by Relion 4"""
         tomoTable = Table(columns=self._getCoordinatesStarFileLabels())
         i = 0
         for coord in coordSet.iterCoordinates():
-            angles, shifts = getTransformInfoFromCoordOrSubtomo(coord)
+            tsId = coord.getTomoId()
+
+            if tsId not in whitelist:
+                continue
+
+            angles, shifts = getTransformInfoFromCoordOrSubtomo(coord, coordSet.getSamplingRate())
             # Add row to the table which will be used to generate the STAR file
             tomoTable.addRow(
-                coord.getTomoId(),  # 1 _rlnTomoName
+                tsId,  # 1 _rlnTomoName
                 coord.getObjId(),  # 2 _rlnTomoParticleId
                 coord.getGroupId() if coord.getGroupId() else 1,  # 3 _rlnTomoManifoldIndex
                 # coord in pix at scale of bin1
@@ -96,9 +101,9 @@ class Writer(WriterTomo):
                 coord.getY(RELION_3D_COORD_ORIGIN) * coordsScale,  # 5 _rlnCoordinateY
                 coord.getZ(RELION_3D_COORD_ORIGIN) * coordsScale,  # 6 _rlnCoordinateZ
                 # pix * Å/pix = [shifts in Å]
-                shifts[0] * sRate,  # 7 _rlnOriginXAngst
-                shifts[1] * sRate,  # 8 _rlnOriginYAngst
-                shifts[2] * sRate,  # 9 _rlnOriginZAngst
+                shifts[0], #* sRate,  # 7 _rlnOriginXAngst
+                shifts[1], #* sRate,  # 8 _rlnOriginYAngst
+                shifts[2], #* sRate,  # 9 _rlnOriginZAngst
                 # Angles in degrees
                 angles[0],  # 10 _rlnAngleRot
                 angles[1],  # 11 _rlnAngleTilt
@@ -107,18 +112,21 @@ class Writer(WriterTomo):
                 int(getattr(coord, '_classNumber', -1)),  # 13_rlnClassNumber
                 # Alternated 1 and 2 values
                 int(getattr(coord, '_randomSubset', (i % 2) + 1)),  # 14 _rlnRandomSubset
-                int(coord.getX(SCIPION)),  # 15 _sciXCoord
-                int(coord.getY(SCIPION)),  # 16 _sciYCoord
-                int(coord.getZ(SCIPION))   # 17 _sciZCoord
+                coord.getX(SCIPION),  # 15 _sciXCoord
+                coord.getY(SCIPION),  # 16 _sciYCoord
+                coord.getZ(SCIPION),   # 17 _sciZCoord
+                coord.getGroupId()  # 18 _sciGroupId
             )
             i += 1
         # Write the STAR file
         tomoTable.write(subtomosStar)
 
-    def pseudoSubtomograms2Star(self, pSubtomoSet, outStar):
+    def pseudoSubtomograms2Star(self, pSubtomoSet, outStar, withPriors=False):
+
+        logger.info("Generating particles file (%s) from pseudosubtomogram set." % outStar)
         sRate = pSubtomoSet.getSamplingRate()
         hasCoords = pSubtomoSet.getFirstItem().hasCoordinate3D()
-        tomoTable = Table(columns=self._getPseudoSubtomogramStarFileLabels(hasCoords))
+        tomoTable = Table(columns=self._getPseudoSubtomogramStarFileLabels(hasCoords, withPriors=withPriors))
 
         # Write the STAR file
         optGroup = OpticsGroups.fromString(pSubtomoSet.getAcquisition().opticsGroupInfo.get())
@@ -128,8 +136,8 @@ class Writer(WriterTomo):
             partsWriter = Table.Writer(f)
             partsWriter.writeTableName(PARTICLES_TABLE)
             partsWriter.writeHeader(tomoTable.getColumns())
-            for pSubtomo in pSubtomoSet:
-                angles, shifts = getTransformInfoFromCoordOrSubtomo(pSubtomo, convention=TR_SCIPION)
+            for pSubtomo in pSubtomoSet.iterSubtomos():
+                angles, shifts = getTransformInfoFromCoordOrSubtomo(pSubtomo, pSubtomo.getSamplingRate())
                 pSubtomoFile = pSubtomo.getFileName()
                 pSubtomoFile = pSubtomoFile.replace(':' + MRC, '') if pSubtomoFile else FILE_NOT_FOUND
                 pSubtomoCtfFile = pSubtomo.getCtfFile() if pSubtomo.getCtfFile() else FILE_NOT_FOUND
@@ -144,9 +152,9 @@ class Writer(WriterTomo):
                     pSubtomo.getY(),  # _rlnCoordinateY #5
                     pSubtomo.getZ(),  # _rlnCoordinateZ #6
                     # pix * Å/pix = [shifts in Å]
-                    shifts[0] * sRate,  # _rlnOriginXAngst #7
-                    shifts[1] * sRate,  # _rlnOriginYAngst #8
-                    shifts[2] * sRate,  # _rlnOriginZAngst #9
+                    shifts[0], #* sRate,  # _rlnOriginXAngst #7
+                    shifts[1], #* sRate,  # _rlnOriginYAngst #8
+                    shifts[2], #* sRate,  # _rlnOriginZAngst #9
                     # Angles in degrees
                     angles[0],  # _rlnAngleRot #10
                     angles[1],  # _rlnAngleTilt #11
@@ -155,10 +163,15 @@ class Writer(WriterTomo):
                     pSubtomo.getClassId(),  # _rlnClassNumber #13
                     pSubtomo.getRdnSubset(),  # _rlnRandomSubset #14
                 ]
+
+                if withPriors:
+                    rowsValues += [angles[0], angles[1], angles[2]]
                 if hasCoords:
                     rowsValues += [pSubtomo.getCoordinate3D().getX(SCIPION),  # _sciXCoord #15
                                    pSubtomo.getCoordinate3D().getY(SCIPION),  # _sciYCoord #16
-                                   pSubtomo.getCoordinate3D().getZ(SCIPION)]  # _sciZCoord #17
+                                   pSubtomo.getCoordinate3D().getZ(SCIPION),  # _sciZCoord #17
+                                   pSubtomo.getCoordinate3D().getGroupId(),   # _sciGroupId #18
+                                   ]
 
                 rowsValues += [pSubtomo.getRe4ParticleName(),  # _rlnTomoParticleName #18
                                pSubtomo.getOpticsGroupId(),  # _rlnOpticsGroup #19
@@ -174,7 +187,7 @@ class Writer(WriterTomo):
         extraPath = join(getParentFolder(subtomosStar), 'extra')
         for subtomo in subtomoSet.iterSubtomos():
             checkSubtomogramFormat(subtomo, extraPath)
-            angles, shifts = getTransformInfoFromCoordOrSubtomo(subtomo)
+            angles, shifts = getTransformInfoFromCoordOrSubtomo(subtomo, subtomo.getSamplingRate())
             ctfFile = getattr(subtomo, '_ctfImage', None)
             if ctfFile:
                 ctfFile = ctfFile.get()
@@ -197,9 +210,9 @@ class Writer(WriterTomo):
             rlnAngleTilt = angles[1]
             rlnAnglePsi = angles[2]
             # pix * Å/pix = [shifts in Å]
-            rlnOriginX = shifts[0] * sRate
-            rlnOriginY = shifts[1] * sRate
-            rlnOriginZ = shifts[2] * sRate
+            rlnOriginX = shifts[0] #* sRate
+            rlnOriginY = shifts[1] #* sRate
+            rlnOriginZ = shifts[2] #* sRate
             # Angles in degrees
             rlnTiltPrior = subtomo._tiltPriorAngle.get() if hasattr(subtomo, '_tiltPriorAngle') else rlnAngleTilt
             rlnPsiPrior = subtomo._psiPriorAngle.get() if hasattr(subtomo, '_psiPriorAngle') else rlnAnglePsi
@@ -238,7 +251,7 @@ class Writer(WriterTomo):
         ]
 
     @staticmethod
-    def _getCoordinatesStarFileLabels(hasCoords=True):
+    def _getCoordinatesStarFileLabels(hasCoords=True, withPriors=False):
         starFileLabels = [
             TOMO_NAME,
             TOMO_PARTICLE_ID,
@@ -256,17 +269,19 @@ class Writer(WriterTomo):
             RANDOM_SUBSET
 
         ]
+
+        if withPriors:
+            starFileLabels+=[ROT_PRIOR, TILT_PRIOR, PSI_PRIOR]
+
         if hasCoords:
-            starFileLabels += [SCIPION_COORD_X, SCIPION_COORD_Y, SCIPION_COORD_Z]
+            starFileLabels += [SCIPION_COORD_X, SCIPION_COORD_Y, SCIPION_COORD_Z, SCIPION_COORD_GROUP_ID]
 
         return starFileLabels
 
     @staticmethod
-    def _getPseudoSubtomogramStarFileLabels(hasCoords=True):
-        pSubtomosLabels = Writer._getCoordinatesStarFileLabels(hasCoords)
+    def _getPseudoSubtomogramStarFileLabels(hasCoords=True, withPriors=False):
+        pSubtomosLabels = Writer._getCoordinatesStarFileLabels(hasCoords, withPriors=withPriors)
         pSubtomosLabels.extend([
-            CLASS_NUMBER,
-            RANDOM_SUBSET,
             TOMO_PARTICLE_NAME,
             OPTICS_GROUP,
             SUBTOMO_NAME,
@@ -361,7 +376,7 @@ class Reader(ReaderTomo):
                     nonMatchingTomoIds += '%s ' % tomoId
 
         if nonMatchingTomoIds:
-            print(yellowStr('The star file contains coordinates that belong to tomograms not present '
+            logger.info(yellowStr('The star file contains coordinates that belong to tomograms not present '
                             'in the introduced set of tomograms: %s' % nonMatchingTomoIds))
 
     def starFile2PseudoSubtomograms(self, outputSet):
@@ -382,8 +397,11 @@ class Reader(ReaderTomo):
                                           rdnSubset=row.get(RANDOM_SUBSET, counter % 2),  # 1 and 2 alt. by default
                                           re4ParticleName=row.get(TOMO_PARTICLE_NAME),
                                           opticsGroupId=row.get(OPTICS_GROUP, 1),
-                                          manifoldIndex=row.get(MANIFOLD_INDEX, 1 if counter % 2 else -1),
-                                          )  # 1 and -1
+                                          manifoldIndex=row.get(MANIFOLD_INDEX, 1 if counter % 2 else -1),  # 1 and -1
+                                          logLikeliCont=row.get(LOG_LIKELI_CONTRIB, -1),
+                                          maxValProbDist=row.get(MAX_VALUE_PROB_DISTRIB, -1),
+                                          noSignifSamples=row.get(NO_SIGNIFICANT_SAMPLES, -1)
+                                          )
 
             # Keeping particle id
             psubtomo.setObjId(row.get(TOMO_PARTICLE_ID))
@@ -394,6 +412,7 @@ class Reader(ReaderTomo):
                 sciCoord.setX(row.get(SCIPION_COORD_X), SCIPION)
                 sciCoord.setY(row.get(SCIPION_COORD_Y), SCIPION)
                 sciCoord.setZ(row.get(SCIPION_COORD_Z), SCIPION)
+                sciCoord.setGroupId(row.get(SCIPION_COORD_GROUP_ID,1))
                 sciCoord.setTomoId(row.get(TOMO_NAME))
                 psubtomo.setCoordinate3D(sciCoord)
 
@@ -402,9 +421,9 @@ class Reader(ReaderTomo):
             psubtomo.setTransform(t)
 
             # Add the files to the list of files whose header has to be corrected to be interpreted as volumes
-            if particleFile:
+            if particleFile is not None and particleFile != FILE_NOT_FOUND:
                 listOfFilesToFixVolume.append(particleFile)
-            if ctfFile:
+            if ctfFile is not None and ctfFile!=FILE_NOT_FOUND:
                 listOfFilesToFixVolume.append(ctfFile)
             # Add current pseudosubtomogram to the output set
             outputSet.append(psubtomo)
@@ -455,59 +474,3 @@ class Reader(ReaderTomo):
         # Set the set of coordinates which corresponds to the current set of subtomograms
         subtomoSet.setCoordinates3D(coordSet)
 
-    def setParticleTransform(self, particle, row, sRate):
-        """ Set the transform values from the row. """
-
-        if (self._alignType == ALIGN_NONE) or not row.hasAnyColumn(self.ALIGNMENT_LABELS):
-            self.setParticleTransform = self.__setParticleTransformNone
-        else:
-            # Ensure the Transform object exists
-            self._angles = np.zeros(3)
-            self._shifts = np.zeros(3)
-
-            particle.setTransform(Transform())
-
-            if self._alignType == ALIGN_2D:
-                self.setParticleTransform = self.__setParticleTransform2D
-            elif self._alignType == ALIGN_PROJ:
-                self.setParticleTransform = self.__setParticleTransformProj
-            else:
-                raise TypeError("Unexpected alignment type: %s"
-                                % self._alignType)
-
-        # Call again the modified function
-        self.setParticleTransform(particle, row, sRate)
-
-    @staticmethod
-    def __setParticleTransformNone(particle, row, sRate):
-        particle.setTransform(None)
-
-    def __setParticleTransform2D(self, particle, row, sRate):
-        angles = self._angles
-        shifts = self._shifts
-
-        shifts[0] = float(row.get(SHIFTX_ANGST / sRate, 0))
-        shifts[1] = float(row.get(SHIFTY_ANGST / sRate, 0))
-        angles[2] = float(row.get(PSI, 0))
-        radAngles = -np.deg2rad(angles)
-        M = tfs.euler_matrix(radAngles[0], radAngles[1], radAngles[2], 'szyz')
-        M[:3, 3] = shifts[:3]
-        particle.getTransform().setMatrix(M)
-
-    def __setParticleTransformProj(self, particle, row, sRate):
-        angles = self._angles
-        shifts = self._shifts
-
-        shifts[0] = float(row.get(SHIFTX_ANGST, 0)) / sRate
-        shifts[1] = float(row.get(SHIFTY_ANGST, 0)) / sRate
-        shifts[2] = float(row.get(SHIFTZ_ANGST, 0)) / sRate
-
-        angles[0] = float(row.get(ROT, 0))
-        angles[1] = float(row.get(TILT, 0))
-        angles[2] = float(row.get(PSI, 0))
-
-        radAngles = -np.deg2rad(angles)
-        M = tfs.euler_matrix(radAngles[0], radAngles[1], radAngles[2], 'szyz')
-        M[:3, 3] = -shifts[:3]
-        M = np.linalg.inv(M)
-        particle.getTransform().setMatrix(M)
