@@ -28,16 +28,17 @@ from os.path import abspath, exists, isfile, join
 
 from emtable import Table
 
+from pyworkflow.protocol import Form
 from pyworkflow.object import Float
 from pyworkflow.utils import moveFile, createLink
-from reliontomo.constants import OUT_PARTICLES_STAR
+from reliontomo.constants import OUT_PARTICLES_STAR, TOMO_PARTICLE_ID
 from reliontomo.objects import RelionSetOfPseudoSubtomograms
 from reliontomo.protocols import ProtRelionRefineSubtomograms
 from reliontomo.protocols.protocol_base_refine import ProtRelionRefineBase
 from reliontomo import Plugin
 from pyworkflow.protocol import FloatParam, BooleanParam, GE, LE, IntParam, StringParam
 from reliontomo.utils import getProgram
-from tomo.objects import SetOfClassesSubTomograms, SetOfAverageSubTomograms
+from tomo.objects import SetOfClassesSubTomograms, SetOfSubTomograms
 
 
 class outputObjects(Enum):
@@ -48,11 +49,12 @@ class outputObjects(Enum):
 class ProtRelion3DClassifySubtomograms(ProtRelionRefineSubtomograms):
     """3D Classification of subtomograms."""
 
-    _label = '3D Classification of subtomograms'
+    _label = '3D classification'
     modelTable = Table()
     classesTable = Table()
     opticsTable = Table()
     particlesTable = Table()
+    _possibleOutputs = outputObjects
 
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
@@ -102,7 +104,7 @@ class ProtRelion3DClassifySubtomograms(ProtRelionRefineSubtomograms):
                            'range of 7-12 Angstroms have proven useful.')
 
     @staticmethod
-    def _defineSamplingParams(form):
+    def _defineSamplingParams(form: Form):
         form.addSection(label='Sampling')
         form.addParam('doImageAlignment', BooleanParam,
                       label='Perform image alignment?',
@@ -110,7 +112,9 @@ class ProtRelion3DClassifySubtomograms(ProtRelionRefineSubtomograms):
                       help='If set to No, then rather than performing both alignment and classification, only '
                            'classification will be performed. This allows the use of very focused masks. It requires '
                            'that the optimal orientations of all particles are already stored in the input STAR file.')
+
         ProtRelionRefineBase._insertAngularCommonParams(form, condition='doImageAlignment')
+
         form.addParam('doLocalAngleSearch', BooleanParam,
                       label='Perform local angular searches?',
                       default=False,
@@ -128,6 +132,7 @@ class ProtRelion3DClassifySubtomograms(ProtRelionRefineSubtomograms):
                            "the optimal orientation in the previous iteration.\n\nA Gaussian prior (also see previous "
                            "option) will be applied, so that orientations closer to the optimal orientation in the "
                            "previous iteration will get higher weights than those further away.")
+
         ProtRelionRefineSubtomograms._insertRelaxSymmetry(form, condition='doImageAlignment and doLocalAngleSearch')
         form.addParam('allowCoarser', BooleanParam,
                       label='Allow coarser sampling?',
@@ -175,14 +180,15 @@ class ProtRelion3DClassifySubtomograms(ProtRelionRefineSubtomograms):
         classes3D = self._createSetOfClassesSubTomograms(relionParticles)
         classes3D.setImages(relionParticles)
         self._fillClassesFromIter(classes3D, self.nIterations.get())
-        averages = SetOfAverageSubTomograms.create(self._getPath(),
-                                                   template='avgPSubtomograms%s.sqlite',
-                                                   suffix='')
-        averages.setSamplingRate(relionParticles.getCurrentSamplingRate())
-        for class3D in classes3D:
-            vol = class3D.getRepresentative()
-            vol.setObjId(class3D.getObjId())
-            averages.append(vol)
+
+        # averages = SetOfAverageSubTomograms.create(self._getPath(),
+        #                                            template='avgPSubtomograms%s.sqlite',
+        #                                            suffix='')
+        # averages.setSamplingRate(relionParticles.getCurrentSamplingRate())
+        # for class3D in classes3D:
+        #     vol = class3D.getRepresentative()
+        #     vol.setObjId(class3D.getObjId())
+        #     averages.append(vol)
 
         outputDict = {outputObjects.relionParticles.name: relionParticles,
                       outputObjects.classes.name: classes3D}
@@ -240,6 +246,7 @@ class ProtRelion3DClassifySubtomograms(ProtRelionRefineSubtomograms):
             cmd += '--offset_step %d ' % (self.offsetSearchStepPix.get() * 2 ** self.oversampling.get())
             if self.doLocalAngleSearch.get():
                 cmd += '--sigma_ang %d ' % (self.localAngularSearchRange.get() / 3)
+
                 if self.relaxSym.get():
                     cmd += '--relax_sym %s ' % self.relaxSym.get()
             if self.allowCoarser.get():
@@ -256,11 +263,12 @@ class ProtRelion3DClassifySubtomograms(ProtRelionRefineSubtomograms):
 
         return cmd
 
-    def _createSetOfClassesSubTomograms(self, subTomograms, suffix=''):
+    def _createSetOfClassesSubTomograms(self, subTomograms: SetOfSubTomograms, suffix=''):
         classes = SetOfClassesSubTomograms.create(self._getPath(),
                                                   template='subtomogramClasses%s.sqlite',
                                                   suffix=suffix)
         classes.setImages(subTomograms)
+        classes.setCoordinates3D(subTomograms.getCoordinates3D(asPointer=True))
         return classes
 
     def _fillClassesFromIter(self, clsSet, iteration):
@@ -283,6 +291,7 @@ class ProtRelion3DClassifySubtomograms(ProtRelionRefineSubtomograms):
         with open(dataStar) as fid:
             self.opticsTable.readStar(fid, 'optics')
             self.particlesTable.readStar(fid, 'particles')
+            self.particlesTable.sort(TOMO_PARTICLE_ID)
 
         # Model table has only one row, while classes table has the same number of rows as classes found
         self.nClasses = int(self.modelTable._rows[0].rlnNrClasses)
@@ -300,9 +309,16 @@ class ProtRelion3DClassifySubtomograms(ProtRelionRefineSubtomograms):
         classId = item.getObjId()
         if classId in self._classesInfo:
             _, row = self._classesInfo[classId]
-            fn = row.rlnReferenceImage + ":mrc"
+            fn = row.rlnReferenceImage
             item.setAlignment3D()
-            item.getRepresentative().setLocation(fn)
+
+            # Representative stuff
+            representative = item.getRepresentative()
+            representative.setLocation(fn)
+            representative.setSamplingRate(self.inReParticles.get().getCurrentSamplingRate())
+            # relion mrc are technically stacks. Fix this
+            representative.fixMRCVolume()
+
             item._rlnclassDistribution = Float(row.rlnClassDistribution)
             item._rlnAccuracyRotations = Float(row.rlnAccuracyRotations)
             item._rlnAccuracyTranslations = Float(row.rlnAccuracyTranslationsAngst)
