@@ -28,7 +28,6 @@ from os import mkdir
 from os.path import join, exists
 from imod.utils import generateDefocusIMODFileFromObject
 from pwem.protocols import EMProtocol
-from pyworkflow import BETA
 from pyworkflow.object import Float
 from pyworkflow.protocol import PointerParam, BooleanParam, LEVEL_ADVANCED
 from pyworkflow.utils import makePath, Message
@@ -38,7 +37,6 @@ from reliontomo.constants import (IN_TOMOS_STAR, OUT_TOMOS_STAR, IN_COORDS_STAR,
                                   OPTIMISATION_SET_STAR, OUT_PARTICLES_STAR, PSUBTOMOS_SQLITE)
 from reliontomo.convert import writeSetOfTomograms, writeSetOfCoordinates, readSetOfPseudoSubtomograms
 from reliontomo.utils import generateProjections
-from tomo.utils import getNonInterpolatedTsFromRelations
 import tomo.objects as tomoObj
 from tomo.protocols.protocol_base import ProtTomoBase
 
@@ -48,6 +46,7 @@ THICKNESS = 'thickness'
 X_SIZE = 'x'
 Y_SIZE = 'y'
 DIMS = 'dims'
+SHIFTS = 'shift'
 
 
 class outputObjects(Enum):
@@ -59,7 +58,6 @@ class ProtRelionPrepareData(EMProtocol, ProtTomoBase):
     """Prepare data for Relion 4
     """
     _label = 'Prepare data for Relion 4'
-    _devStatus = BETA
     _possibleOutputs = outputObjects
 
     def __init__(self, **args):
@@ -67,17 +65,21 @@ class ProtRelionPrepareData(EMProtocol, ProtTomoBase):
         self.tsSet = None
         self.tomoSet = None
         self.tsReducedSet = None  # Reduced list of tiltseries with only those in the coordinates
-        self.coordsVolIds = None  # Unique tomogram identifiers volId used in the coordinate set. In case is a subset
+        self.matchingTSIds = None  # Unique tomogram identifiers volId used in the coordinate set. In case is a subset
         self.coordScale = Float(1)
 
     # -------------------------- DEFINE param functions -----------------------
 
     def _defineParams(self, form):
         form.addSection(label=Message.LABEL_INPUT)
-
+        form.addParam('inputCoords', PointerParam,
+                      pointerClass='SetOfCoordinates3D',
+                      label="Coordinates",
+                      important=True,
+                      allowsNull=False)
         form.addParam('inputCtfTs', PointerParam,
                       pointerClass='SetOfCTFTomoSeries',
-                      label="Input CTF tomo series",
+                      label="CTF tomo series",
                       important=True,
                       allowsNull=False)
         form.addParam('handeness', BooleanParam,
@@ -86,28 +88,19 @@ class ProtRelionPrepareData(EMProtocol, ProtTomoBase):
                       expertLevel=LEVEL_ADVANCED,
                       help='It is the handedness of the tilt geometry and it is used to describe '
                            'whether the focus increases or decreases as a function of Z distance.')
-        form.addParam('inputCoords', PointerParam,
-                      pointerClass='SetOfCoordinates3D',
-                      label="Input coordinates",
+        form.addParam('inputTS', PointerParam,
+                      help="Tilt series with alignment (non interpolated) used in the tomograms reconstruction.",
+                      pointerClass='SetOfTiltSeries',
+                      label="Tilt series",
                       important=True,
                       allowsNull=False)
-        form.addParam('inputTS', PointerParam,
-                      help="Tilt series with alignment (non interpolated) used in the tomograms recnstruction. "
-                           "To be deprecated!!",
-                      pointerClass='SetOfTiltSeries',
-                      label="Input tilt series",
-                      important=True,
-                      allowsNull=True)
-
         form.addParam('flipZCoords', BooleanParam,
                       label='Flip Z coordinate?',
                       default=False,
                       help='This option is generally False if your coordinates are displayed correctly in Scipion. '
                            'You may want to check this to True only if you see that the extracted subtomograms'
                            ' are wrong.',
-                      expertLevel=LEVEL_ADVANCED
-                      )
-
+                      expertLevel=LEVEL_ADVANCED)
         form.addParam('flipYZ', BooleanParam,
                       label='Has tomogram been flipped along Y and Z?',
                       default=True,
@@ -121,7 +114,6 @@ class ProtRelionPrepareData(EMProtocol, ProtTomoBase):
                       default=True,
                       help='This option is generally True when you apply reconstrucion in Imod. This is usually used '
                            'together with the flipYZ option.')
-
         form.addParam('swapXY', BooleanParam,
                       label='Swap X with Y dimensions of the tilt series',
                       default=False,
@@ -144,8 +136,22 @@ class ProtRelionPrepareData(EMProtocol, ProtTomoBase):
         if not exists(defocusDir):  # It can exist in case of Continue execution
             mkdir(defocusDir)
         self.coords = self.inputCoords.get()
-        self.coordsVolIds = self._getTSIDFromCoordinates()
         self.tsSet = self._getTiltSeriesNonInterpolated()
+
+        # Compute matching TS id among coordinates and tilt series, both could be a subset
+        coordsTsIds = self.coords.getTSIds()
+        self.info("TsIds present in coordinates are: %s" % coordsTsIds)
+
+        tsIds = self.tsSet.getTSIds()
+        self.info("TsIds present in Tilt series are: %s" % tsIds)
+
+        # Intersection
+        self.matchingTSIds = set(coordsTsIds).intersection(tsIds)
+        if len(self.matchingTSIds):
+            self.info("Tilt series associated in both, coordinates and tilt series are %s" % self.matchingTSIds)
+        else:
+            raise Exception("There isn't any common tilt series among the coordinates and tilt series chosen.")
+
         self.tomoSet = self.coords.getPrecedents()
         self.inputCtfs = self.inputCtfTs.get()
         # If coordinates are referred to a set of tomograms, they'll be rescaled
@@ -153,39 +159,51 @@ class ProtRelionPrepareData(EMProtocol, ProtTomoBase):
         if self.tomoSet:
             self.coordScale.set(self.tomoSet.getSamplingRate() / self.tsSet.getSamplingRate())
 
-    def _getTiltSeriesNonInterpolated(self):
-        return self.inputTS.get() if self.inputTS.get() is not None else \
-            getNonInterpolatedTsFromRelations(self.inputCoords.get(), self)
-
-    def _getTSIDFromCoordinates(self):
-        # TODO: Add this functionality to the SetOFCoordinates. May be useful for other cases
-        volIds = self.coords.aggregate(["MAX"], "_tomoId", ["_tomoId"])
-        volIds = [d['_tomoId'] for d in volIds]
-        self.info("Involved tomograms in the coordinates: %s" % volIds)
-        return volIds
+    def _getTiltSeriesNonInterpolated(self) ->tomoObj.SetOfTiltSeries:
+        return self.inputTS.get() # if self.inputTS.get() is not None else \
+            #getNonInterpolatedTsFromRelations(self.inputCoords.get(), self)
 
     def convertInputStep(self):
         # Generate defocus files
         for ctfTomo in self.inputCtfs:
+            tsId = ctfTomo.getTsId()
+            if tsId not in self.matchingTSIds:
+                continue
             defocusPath = self._getExtraPath(DEFOCUS, ctfTomo.getTsId())
             if not exists(defocusPath):  # It can exist in case of mode Continue execution
                 mkdir(defocusPath)
             generateDefocusIMODFileFromObject(ctfTomo,
                                               join(defocusPath, ctfTomo.getTsId() + '.' + DEFOCUS),
                                               isRelion=True)
-        # Thickness of the tomogram
+        # Thickness of the tomogram and shifts
         tomoSizeDict = {}
+        tomoShiftsDict ={}
         tomoList = [tomo.clone() for tomo in self.coords.getPrecedents()]
         coordScale = self.coordScale.get()
         for tomo in tomoList:
+            tsId = tomo.getTsId()
+            if tsId not in self.matchingTSIds:
+                continue
+
             x, y, thickness = tomo.getDim()
-            tomoSizeDict[tomo.getTsId()] = {X_SIZE: x * coordScale,
+            tomoSizeDict[tsId] = {X_SIZE: x * coordScale,
                                             Y_SIZE: y * coordScale,
                                             THICKNESS: thickness * coordScale}
 
+            # Based on this: https://github.com/3dem/relion/blob/ver4.0/src/jaz/tomography/programs/convert_projections.cpp#L368
+            # First element is X, second Z!
+            shiftsAngs = tomo.getShiftsFromOrigin()
+
+            # shifts are stored in Angstrom, we convert to tomo SR and then add the half and the to TS pixel size
+            shiftX = int(((shiftsAngs[0] / tomo.getSamplingRate()) + x/2) * self.coordScale.get())
+            shiftZ = int(((shiftsAngs[2] / tomo.getSamplingRate()) + thickness/2) * self.coordScale.get())
+            tomoShiftsDict[tsId] = (shiftX, shiftZ)
+
+            self.info("Shifts detected for %s are: %s" % (tomo.getTsId(), (shiftX, shiftZ)))
+
         # Simulate the etomo files that serve as entry point to relion4
-        self._simulateETomoFiles(self.tsSet, tomoSizeDict, binned=1, binByFactor=self.coordScale,
-                                 whiteList=self.coordsVolIds, swapDims=self.swapXY.get())
+        self._simulateETomoFiles(self.tsSet, tomoSizeDict, tomoShiftsDict, binned=1, binByFactor=self.coordScale,
+                                 whiteList=self.matchingTSIds, swapDims=self.swapXY.get(), tltIgnoresExcluded=True)
 
         # Write the tomograms star file
         writeSetOfTomograms(self.tsSet,
@@ -193,10 +211,11 @@ class ProtRelionPrepareData(EMProtocol, ProtTomoBase):
                             prot=self,
                             ctfPlotterParentDir=self._getExtraPath(DEFOCUS),
                             eTomoParentDir=self._getTmpPath(),
-                            whiteList=self.coordsVolIds)
+                            whiteList=self.matchingTSIds)
         # Write the particles star file
         writeSetOfCoordinates(self.inputCoords.get(),
                               self._getStarFilename(IN_COORDS_STAR),
+                              self.matchingTSIds,
                               sRate=self.tsSet.getSamplingRate(),
                               coordsScale=self.coordScale.get())
 
@@ -211,7 +230,7 @@ class ProtRelionPrepareData(EMProtocol, ProtTomoBase):
         coords=self.inputCoords.get()
         coordSize = coords.getBoxSize()
         tsSamplingRate = self.tsSet.getSamplingRate()
-        coordSizeAng = coordSize / coords.getSamplingRate()
+        fiducialSize = int((coordSize * coords.getSamplingRate()) / (2*10))  # Radius in nm
 
         psubtomoSet = createSetOfRelionPSubtomograms(self._getPath(),
                                                      self._getExtraPath(OPTIMISATION_SET_STAR),
@@ -234,11 +253,14 @@ class ProtRelionPrepareData(EMProtocol, ProtTomoBase):
 
         fiducialModelGaps = self._createSetOfLandmarkModels(suffix='Gaps')
         fiducialModelGaps.copyInfo(self.tsSet)
-        fiducialModelGaps.setSetOfTiltSeries(self.tsSet)
+        fiducialModelGaps.setSetOfTiltSeries(self.inputTS) # Use the pointer better when scheduling
 
         pos = 0
         for ts in self.tsSet:
             tsId = ts.getTsId()
+
+            if tsId not in self.matchingTSIds:
+                continue
             landmarkModelGapsFilePath = os.path.join(self._getExtraPath(),
                                                      str(tsId) + "_gaps.sfid")
 
@@ -246,15 +268,18 @@ class ProtRelionPrepareData(EMProtocol, ProtTomoBase):
                                                       tiltSeriesPointer=ts,
                                                       fileName=landmarkModelGapsFilePath,
                                                       modelName=None,
-                                                      size=coordSizeAng,
+                                                      size=fiducialSize,
                                                       applyTSTransformation=False)
             landmarkModelGaps.setTiltSeries(ts)
 
-            while pos < len(projections) and projections[pos][0] == tsId:
-                tiltIm = projections[pos][1] + 1
-                chainId = projections[pos][2] + 1
-                xCoor = int(round(projections[pos][3]))
-                yCoor = int(round(projections[pos][4]))
+            # Get the projections for the tilt series
+            tsProjections = projections[tsId]
+
+            for projection in tsProjections:
+                tiltIm = projection[1] + 1
+                chainId = projection[2] + 1
+                xCoor = int(round(projection[3]))
+                yCoor = int(round(projection[4]))
                 landmarkModelGaps.addLandmark(xCoor, yCoor, tiltIm,
                                               chainId, 0, 0)
                 pos += 1
@@ -264,18 +289,11 @@ class ProtRelionPrepareData(EMProtocol, ProtTomoBase):
         self._defineSourceRelation(self.tsSet, fiducialModelGaps)
 
     # -------------------------- INFO functions -------------------------------
-    def _validate(self):
-        # TODO: generar los nombres culled --> tsId_culled.st:mrc cuando se quiten vistas con IMOD
-        valMsg = []
-
-        if not self.inputTS.get():
-            try:
-                self._getTiltSeriesNonInterpolated()
-            except:
-                valMsg.append('Unable to go via relations from the introduced coordinates to the '
-                              'corresponding non-interpolated tilt series. Please introduce them using the '
-                              'advanced parameter "Tilt series with alignment..."')
-        return valMsg
+    def _warnings(self):
+        warnMsg = []
+        if not (self.inputTS.get().hasAlignment() and not self.inputTS.get().interpolated()):
+            warnMsg.append('The introduced tilt series do not have an alignment transformation associated.')
+        return warnMsg
 
     def _summary(self):
         msg = []
@@ -316,7 +334,7 @@ class ProtRelionPrepareData(EMProtocol, ProtTomoBase):
     def _decodeHandeness(self):
         return -1 if self.handeness.get() else 1
 
-    def _simulateETomoFiles(self, imgSet, tomoSizeDict, whiteList=None, **kwargs):
+    def _simulateETomoFiles(self, imgSet, tomoSizeDict, tomoShiftDict, whiteList=None, **kwargs):
         """Simulate the etomo files that serve as entry point to relion4
         """
         for ts in imgSet:
@@ -326,6 +344,10 @@ class ProtRelionPrepareData(EMProtocol, ProtTomoBase):
             if tomoIdMatchDict and (whiteList is None or tsId in whiteList):
                 kwargs[THICKNESS] = tomoIdMatchDict[THICKNESS]
                 kwargs[DIMS] = (tomoIdMatchDict[X_SIZE], tomoIdMatchDict[Y_SIZE])
+
+                # Reconstruction shifts
+                kwargs[SHIFTS] = tomoShiftDict.get(tsId, (0,0))
+
                 # creating a folder where all data will be generate
                 folderName = self._getTmpPath(tsId)
                 makePath(folderName)
